.editbox {margin: .4em;padding: 0;font-family: monospace;font-size: 10pt;color: black;}
span.js-keyword {color: #90b;}
span.js-atom {color: #291;}
span.js-variabledef {color: #00f;}
span.js-localvariable {color: #049;}
span.js-comment {color: #a70;}
span.js-string {color: #a22;}
